﻿/**
 * @file	ColourPalette.cs
 * @brief	Implements the colour palette class.
 * @author	Michael Lofgren
 * @date	2/4/2012
 * $LastChangedDate: 2012-09-05 17:45:44 +1000 (Wed, 05 Sep 2012) $
 * $LastChangedRevision: 2737 $
 * $LastChangedBy: lof012 $
 * $HeadURL: https://svnserv.csiro.au/svn/Nuclear/code/software/xNL/xNL_Analyser/trunk/Source/ColourPalette.cs $
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

using log4net;
using log4net.Config;


namespace xNL_Analyser {

	/**
	 * @brief	Colour palette. 
	 * @details	
	 * @version	3/4/2012 Michael Lofgren
	 *          Initial implementation
	 * @sa	System.Object
	 */
	public class ColourPalette {

      private static readonly ILog log = LogManager.GetLogger("ColourPalette");

		/**
		 * @brief	Default constructor
		 */
		public ColourPalette()
		{
			mColours = new List<System.Drawing.Color>();
		}

		public List<System.Drawing.Color> Colours { get { return mColours; } }
		private List<System.Drawing.Color> mColours;		//!< The colors of the colour map

		public bool Load(String fname)
		{
			int red, green, blue; // Red, green, blue colour intensities

			XmlReader reader;
			XmlReaderSettings settings = new XmlReaderSettings();
			settings.ConformanceLevel = ConformanceLevel.Fragment;
			settings.IgnoreWhitespace = true;
			settings.IgnoreComments = true;

			try {
				reader = XmlReader.Create(fname, settings);

				reader.Read();
				reader.ReadStartElement("Palette");

				reader.ReadToDescendant("item");

				do {
					String col = reader.GetAttribute("color");
					//Trace.WriteLine(col);

					// Now split into red, green, blue, and parse/convert hex value to decimal
					
					red = Convert.ToInt32(col.Substring(1, 2), 16);
					green = Convert.ToInt32(col.Substring(3, 2), 16);
					blue = Convert.ToInt32(col.Substring(5, 2), 16);

					// and add the color to the Colors list
					mColours.Add(Color.FromArgb(red, green, blue));

				} while (reader.ReadToNextSibling("item")); 

			} catch (Exception e) {
            log.Error("While loading colour palette:", e);
            //Trace.WriteLine(e.ToString());
				ErrorNotifyBox.Show(e.ToString());
			}

			return true;
		} // end Load()

		/**
		 * @brief	Gets the palette from the supplied configuration file
		 * @param	cfgfilename	The configuration file name
		 * @return	The palette from the configuration file
		 */
		public static ColourPalette GetPaletteFromConfigFileName(String cfgfilename)
		{
			ColourPalette colourPalette = new ColourPalette();

			string cmapfilename = "";
			// Try and read the colour map assuming ColourPaletteFile is an absolute path to file.
			if (File.Exists(cfgfilename)) {
				cmapfilename = cfgfilename;

				// Now try it as a path relative to the executable's path
			} else {
				String exedir = Path.GetDirectoryName(Application.ExecutablePath);
				String temppathname = Path.Combine(exedir, cfgfilename);
				if (File.Exists(temppathname)) {
					cmapfilename = temppathname;
				}
			}
			colourPalette.Load(cmapfilename);

			return colourPalette;
		}

	} // end class ColourPalette
}
